---------------------------------------------------------------------------
-- Spellsteal Monitor
-- Creates a small frame with stealable spells from your current target.
-- Blinddate (Us-Thrall)
-- Alfabravo (US-Manoroth)
-- Iniitial release commit 01/16/10
-- 10/13/10 TOC bump for 4.01
-- 1.2.. Compatible with Cataclysm, frame autosizes to number of stealable buffs, click frame to cast spellsteal.
-- 1.1 -- .toc bump. 
---------- Thanks to b_buck (@Curse) creator of MageNuggets for the inspiration and some of the code used in this  :)


-- 01/01/2016
-- SpellSteal wasn't being maintained, so I have robbed it blind and added my own wishes and wants
-- Ishkulin, Chamber of Aspects (EU)

-- 1.2 - Typo correction causing lua error
-- 1.3 - Only shows on a Mage now
--       Added TimeWarp notice, we don't want the blame when it isn't us!!
-- 1.4 - whisper to self, as requested by shinjiat
-- 1.5 - MOVE command line command added for datgrl :) (eventually)
---------------------------------------------------------------------------
local version = GetAddOnMetadata("SpellThief", "Version")

STSettings = 
	{
		STEnabled = true,
		STPopup = false,
		STChannels = { 1,1,1,1 },
		STSound = true
	}

STChannelList = { nil,"WHISPER","SAY","YELL","GUILD","OFFICER","INSTANCE_CHAT","PARTY","RAID","RAID_WARNING" }
STChannelListText = { "None","Whisper","Say","Yell","Guild","Officer","Instance","Party","Raid","Raid warning" }
STIsLoaded = false 
STToSteal = 0
STFrame = nil
STFrameList = nil
STSliderRaid = nil
STSliderInstance = nil
STSliderParty = nil
STSliderSolo = nil
STTesting = false
STAnnounced = 0 

function ST_handleCmd(msg, editbox)
	if (msg == "test" ) then
		if (STTesting == true) then
			STTesting = false
			DEFAULT_CHAT_FRAME:AddMessage("Spell Thief: Disabling test frame.")
			STFrameUpdate()
		else
			STTesting = true
			PlaySound("AuctionWindowOpen")
			STFrameUpdate()
			DEFAULT_CHAT_FRAME:AddMessage("Spell Thief: Showing test frame.")
		end
	elseif (msg == "announce") then
		if (STSettings.STAnnounce == true) then
			DEFAULT_CHAT_FRAME:AddMessage("Spell Thief: Announcing to raid/party disabled.")
			STSettings.STAnnounce = false
			if (not STFrametitle) then
				STFrameCreate()
				STFrameUpdate()
			end
		else
			DEFAULT_CHAT_FRAME:AddMessage("Spell Thief: Announcing to raid/party enabled.")
			STSettings.STAnnounce  = true
			if (not STFrametitle) then
				STFrameCreate()
				STFrameUpdate()
			end
		end
		ST_StealingAnnounce:SetChecked(STAnnounce == true)
		ST_SetFrameTitle()
	elseif (msg == "lock" ) then
		if (STFrame.Locked == true) then
			STFrame.Locked = false
			DEFAULT_CHAT_FRAME:AddMessage("Spell Thief: Frame is now unlocked. Frame will auto lock if you reload UI or restart game.")
		else
			STFrame.Locked = true
			STTesting = false
			DEFAULT_CHAT_FRAME:AddMessage("Spell Thief: Frame is now locked.")
		end
			STFrameUpdate()
	elseif (msg == "growup") then
		if (STSettings.STGrowup == true) then
			STSettings.STGrowup = false
			DEFAULT_CHAT_FRAME:AddMessage("Spell Thief: Frame will now grow down (normal).")
			STFrameList:ClearAllPoints()
			STFrameList:SetPoint("TOPLEFT",0,-21)
			STFrameUpdate()
		else
			STSettings.STGrowup = true
			DEFAULT_CHAT_FRAME:AddMessage("Spell Thief: Frame will now grow up (reverse).")
			STFrameList:ClearAllPoints()
			STFrameList:SetPoint("BOTTOMLEFT",0,21)
			STFrameUpdate()
		end
		ST_StealingDirection:SetChecked(STGrowup == true)
	elseif (msg == "move") then
			STTesting = true
			PlaySound("AuctionWindowOpen")
			STFrame.Locked = false
			DEFAULT_CHAT_FRAME:AddMessage("Spell Thief: Frame is now unlocked and moveable, use \"/st lock\" to lock the frame in position again after moving it")
			STFrameUpdate()
	else
		DEFAULT_CHAT_FRAME:AddMessage("Spell Thief: The following commands are recognized. \n\r\"test\" -- Shows Spell Thief frame for positioning.\n\r\"announce\" -- Toggles on/off announcing spells stolen to raid/party.\r\n\"lock\" --Toggles on/off the frame locking. (Defaults to locked)\r\n\"growup\" -- Toggles the frame growing down (default) or up (reversed)\r\n\"move\" -- unlocks the frame for you to move it")
		
	end
end


function ST_OnLoad(self)
	local _, playerClass = UnitClass("player")
	
	if (playerClass == "MAGE") then
		self:RegisterEvent("PLAYER_TARGET_CHANGED")
		self:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
		self:RegisterEvent("UNIT_AURA")	
		self:RegisterEvent("ADDON_LOADED")	
		self:RegisterEvent("PLAYER_DEAD")
		self:RegisterEvent("UNIT_TARGET")
		SLASH_SPELLTHIEF1 = "/sthief"
		SLASH_SPELLTHIEF2 = "/st"
		SlashCmdList["SPELLTHIEF"] = ST_handleCmd
	
		STFrameCreate()
		STFrameUpdate()
	end
	
end

function round(num, idp)
  local mult = 10^(idp or 0)
  return math.floor(num * mult + 0.5) / mult
end

function STFrameUpdate()

	if (STSettings.STEnabled == false or STIsLoaded == false) then
		return
	end

	if not STFrame then
		STFrameCreate()
	end

	local i = 1
	local j = 1
	local stealableBuffs = { }
	local buffName, _, _, _, _, _, expireTime, _, isStealable = UnitAura("target", i, "HELPFUL")

	while buffName do
		if (isStealable == true) then
			if (expireTime) then
				expireTime = round(expireTime - GetTime(),1)
				if (expireTime > 60) then
					expireTime = ""
				else
					expireTime = expireTime .. "s"
				end
			end
			stealableBuffs[j] = buffName .. " " .. expireTime
			j = j + 1
		end
		i = i + 1
		buffName, _, _, _, _, _, expireTime, _, isStealable = UnitAura("target", i, "HELPFUL")
	end
	
	j = j - 1
	
	if (j > 0 and STSettings.STSound == true and not (j == STToSteal)) then
		PlaySound("AuctionWindowOpen")
	end
	STToSteal = j

	if (STSettings.STPopup == true and STTesting == false and j == 0) then
		STFrame:Hide()
	else
		local height = (10 * j) + 8
		stealableBuffs = table.concat(stealableBuffs, "\n")
		STFrameList:SetHeight(height)
		STFrameList.DisplayText:SetText(stealableBuffs)
		if (STSettings.STGrowup == true) then
			STFrameList:ClearAllPoints()
			STFrameList:SetPoint("BOTTOMLEFT",0,25)
		else
			STFrameList:ClearAllPoints()
			STFrameList:SetPoint("TOPLEFT",0,-25)
		end

		STFrame:Show()
	end
end

function ST_SetFrameTitle()
	if (STSettings.STAnnounce == true) then
		STFrametitle:SetText("Spell Thief (Announce mode)")
	else
		STFrametitle:SetText("Spell Thief")
	end
end

function ST_handleEvent(self, event, ...)
local arg1 = ...

	if (STSliderRaid == nil and event == "ADDON_LOADED" and arg1 == "SpellThief") then
		STSliderRaid = CreateFrame("Slider", "STSliderRaidUI", ST_StealingConfigFrame, "OptionsSliderTemplate")
		ST_SetupSlider(ST_StealingConfigFrame,STSliderRaid,-140,STSettings.STChannels[1],10,"Raid")
		ST_SetToolTip(STSliderRaid,"Set channel to announce to when in a raid")
		STSliderInstance = CreateFrame("Slider", "STSliderInstanceUI", ST_StealingConfigFrame, "OptionsSliderTemplate")
		ST_SetupSlider(ST_StealingConfigFrame,STSliderInstance,-170,STSettings.STChannels[2],8,"Grouped")
		ST_SetToolTip(STSliderInstance,"Set channel to announce to when in a instance")
		STSliderParty = CreateFrame("Slider", "STSliderPartyUI", ST_StealingConfigFrame, "OptionsSliderTemplate")
		ST_SetupSlider(ST_StealingConfigFrame,STSliderParty,-200,STSettings.STChannels[3],8,"Party")
		ST_SetToolTip(STSliderParty,"Set channel to announce to when in a party")
		STSliderSolo = CreateFrame("Slider", "STSliderSoloUI", ST_StealingConfigFrame, "OptionsSliderTemplate")
		ST_SetupSlider(ST_StealingConfigFrame,STSliderSolo,-230,STSettings.STChannels[4],6,"Solo")	
		ST_SetToolTip(STSliderSolo,"Set channel to announce to when solo (Awww, Johnny No Mates!)")
		ST_SetFrameTitle()
		STIsLoaded = true
		if (STSettings.STPopup == true) then
			STFrame:Hide()
		end
		STFrameUpdate()
	end

	if (STIsLoaded == false) then return end
	
	local isParty = ((GetNumSubgroupMembers() >0) and not IsInRaid())
	local isGrouped = IsInGroup()
	
	local nChan = 1
	local channel = nil
	
	if (STSettings.STAnnounce == true) then
		if (IsInRaid()) then
			nChan = STSliderRaid:GetValue()
		elseif (isGrouped == true) then
			nChan = STSliderInstance:GetValue()
		elseif (isParty) then
			nChan = STSliderParty:GetValue()
		else
			nChan = STSliderSolo:GetValue()
		end
	end
	
	channel = STChannelList[nChan]
	
	--SPELL_STOLEN,0x05000000045E6EDB,"Alfabravo",0x511,0x0,0x06800000006AC840,"Pison-Drenden",0x10548,0x0,21562,"Power Word: Fortitude",0x2,0x06800000006AC840,63325,48,9789,0,186685,30449,"Spellsteal",64,BUFF
	
	if (event == "PLAYER_TARGET_CHANGED") then
		STFrameUpdate()
	elseif(event == "UNIT_TARGET" and select(1,...) == "player") then
		STFrameUpdate()
	elseif (event == "PLAYER_DEAD") then
		STFrameUpdate()
	elseif (event == "COMBAT_LOG_EVENT_UNFILTERED")	 then
		local cEvent, _, sourceGUID, sourceName, _,_, destGUID,destName,_,_,nid = select(2, ...)
		local spellID = select(15,...)
		local player = UnitGUID("player")
		local ourname = UnitName("player")
		
		if (cEvent == "SPELL_STOLEN" and sourceGUID == player) then
			
			local msg = "Stole:"..GetSpellLink(spellID)
			local name, _, icon, _,_, _, _, _, _ = GetSpellInfo(spellID)
			
			if(GetCVar("enableCombatText") == '1') then
				CombatText_AddMessage(msg, CombatText_StandardScroll, 0.10, 0, 1, "sticky", nil);
			end
			if MikSBT then
				MikSBT.DisplayMessage(msg,MikSBT.DISPLAYTYPE_NOTIFICATION, true, 255, 255, 255, nil, nil, nil, icon)
			end
			if SCT then
				local rgbcolor = { r=1, g=1, b=1 };
				SCT:DisplayMessage(msg, rgbcolor);
			end
  			if Parrot then
				Parrot:ShowMessage(msg, "Notification", true, 1, 1, 1, nil, nil, "NORMAL", icon);
			end
			
			if (channel ~= nil) then
				if (nChan == 2) then
					SendChatMessage("We stole "..GetSpellLink(spellID).. " from "..destName,"WHISPER",nil,ourname)
				else
					msg = "I have thieved "..GetSpellLink(spellID).." from "..destName
					SendChatMessage(msg, channel)
				end
			else
				DEFAULT_CHAT_FRAME:AddMessage("|cffFFFFFFYou have stolen "..GetSpellLink(spellID).." from "..destName)
			end
				
			STFrameUpdate()
		elseif (cEvent == "SPELL_AURA_APPLIED" and nid == 80353 and destGUID == player) then --timewarp
			local tt = time()
			if (tt - STAnnounced > 60) then -- limit these announcements, saves couting players etc to prevent multiple msgs
				STAnnounced = tt
				if (sourceGUID == player) then
					DEFAULT_CHAT_FRAME:AddMessage("I have cast TimeWarp!")
				else
					DEFAULT_CHAT_FRAME:AddMessage(sourceName .. " has cast TimeWarp!")
				end
			end
	    end
	elseif (event == "UNIT_AURA" and select(1,...) == "target") then
		STFrameUpdate()
	end
end


function STFrameCreate()
   
	local backdrop = {bgFile = "Interface\\Tooltips\\UI-Tooltip-Background",edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border", edgeSize = 3, left=2, right=2, top=2, bottom=2}
	
	if not STFrame then
		STFrame = CreateFrame("Frame", "STFrame", UIParent)
		STFrame:SetClampedToScreen(true)
		STFrame:SetFrameStrata("HIGH")
		STFrame:SetBackdrop(backdrop)
		STFrame:SetBackdropColor(0,0,0,1)
		STFrame:SetWidth(220)
		STFrame:SetHeight(20) 
		STFrame:SetPoint("CENTER",0,0)
		STFrame:EnableMouse(true)
		STFrame:SetMovable(true)
		STFrame:RegisterForDrag("RightButton")
		STFrame:SetUserPlaced(true)
		
		STFrame.Locked = true
			
		STFrame:SetScript('OnMouseDown', function(self) 
			if (self.Locked == false) then
				self:StartMoving()
				self.IsMoving = true
			end
		end)
		STFrame:SetScript('OnMouseUp', function(self)
			if (self.Locked == false) then
				self:StopMovingOrSizing()
				self.IsMoving = false
			end
		end)
		STFrame:SetScript("OnShow", function(self)
			STFrameList:Show()
		end)
		
		STFrame:SetScript("OnHide", function(self)
			if ( self.isMoving ) then
				self:StopMovingOrSizing();
				self.isMoving = false;
			end
		end)
		
		STFrametitle = STFrame:CreateFontString("STFrametitletext", "OVERLAY")
		STFrametitle:SetFont("Fonts\\FRIZQT__.TTF", 12)
		STFrametitle:SetJustifyH("LEFT")

		ST_SetFrameTitle()
		STFrametitle:SetPoint("TOPLEFT", 0, -4)
		
		STFrameList = CreateFrame("Frame", "STFrameList", STFrame)
		STFrameList:SetFrameStrata("HIGH")
		
		if ( STGrowup == true) then
			STFrameList:SetPoint("BOTTOMLEFT",0,21)
		else
			STFrameList:SetPoint("TOPLEFT",0,-21)
		end
		
		STFrameList:SetWidth(220)
		STFrameList:SetHeight(80)
		STFrameList:SetBackdrop(backdrop)
		STFrameList:SetBackdropColor(1,0,0,.5)
		STFrameList:SetBackdropBorderColor(0,0,0)
		STFrameList.elapsed = 0
		STFrameList.DisplayText = STFrameList:CreateFontString("STFrameListText", "OVERLAY", STFrameList)
		STFrameList.DisplayText:SetFont("Fonts\\FRIZQT__.TTF", 10)
		STFrameList.DisplayText:SetJustifyH("LEFT")
		STFrameList.DisplayText:SetPoint("LEFT")
	end
	
end

function ST_Stealing_toggleEnable()
	STSettings.STEnabled = not STSettings.STEnabled
end
function ST_Stealing_toggleDirection()
	STSettings.STGrowup = not STSettings.STGrowup
	STFrameUpdate()
end
function ST_Stealing_toggleAnnounce()
	STSettings.STAnnounce = not STSettings.STAnnounce
	ST_SetFrameTitle()
end
function ST_Stealing_togglePopup()
	STSettings.STPopup = not STSettings.STPopup
	if (STSettings.STPopup == true) then
		STFrame:Hide()
	else
		STFrame:Show()
	end
end
function ST_Stealing_toggleSound()
	STSettings.STSound = not STSettings.STSound
end

function ST_SetSliderText(slider)
	local val = slider:GetValue()
	local str = STChannelListText[val]
	local slid = slider:GetName() .. '_tag'
	_G[slid]:SetText(str)
	
	if (slider == STSliderRaid) then STSettings.STChannels[1] = val
		elseif (slider == STSliderInstance) then STSettings.STChannels[2] = val
		elseif (slider == STSliderParty) then STSettings.STChannels[3] = val
		elseif (slider == STSliderSolo) then STSettings.STChannels[4] = val
	end
end

function ST_SetupSlider(parent,slider,y,value,top,pre)
	slider:SetMinMaxValues(1,top)
	slider:SetWidth(200)
	slider:SetHeight(30)
	slider:SetPoint("TOPLEFT",parent,"TOPLEFT",80,y)
	slider:SetValueStep(1)
	slider:SetObeyStepOnDrag(true)
	getglobal(slider:GetName() .. 'Low'):SetText('')
	getglobal(slider:GetName() .. 'High'):SetText('')
	
	fs = parent:CreateFontString()
	fs:SetFont("Fonts\\FRIZQT__.TTF", 12)
	fs:SetText(pre)
	fs:SetPoint("TOPLEFT",parent,"TOPLEFT",70-fs:GetStringWidth(),y-8)

	fs2 = parent:CreateFontString(slider:GetName() .. '_tag')
	fs2:SetFont("Fonts\\FRIZQT__.TTF", 12)
	fs2:SetPoint("TOPLEFT",parent,"TOPLEFT",300,y-8)
	fs2:SetText(pre)
	
	slider:SetScript("OnValueChanged",ST_SetSliderText)
	slider:SetValue(value)
end

function ST_StealingConfigFrame_OnLoad(panel)
	local _, playerClass = UnitClass("player")
	
	if (playerClass == "MAGE") then
		panel.name = "Spell Thief" ;
		InterfaceOptions_AddCategory(panel);
	end
end

function ST_OnEnterElement(self)
	GameTooltip:SetOwner(self, "ANCHOR_RIGHT")
	GameTooltip:SetText("Spell Thief", 1.0, 0.0, 1.0, nil, true)
	GameTooltip:AddLine("-----------", 1.0, 0.0, 1.0, nil, true)
	GameTooltip:AddLine(self.tooltipText, nil, nil, nil, nil, true)
	GameTooltip:Show()
end

function ST_OnLeaveElement()
	GameTooltip_Hide()
end

function ST_SetToolTip(self, text)
	self.tooltipText = "|cFFFFFFFF" .. text
	self:SetScript("OnEnter", ST_OnEnterElement)
	self:SetScript("OnLeave", ST_OnLeaveElement)
end

function ST_StealingConfigFrame_OnShow()
	ST_StealingDirection:SetChecked(STSettings.STGrowup == true)
	ST_SetToolTip(ST_StealingDirection,"If ticked, then the box grows up")
	ST_StealingAnnounce:SetChecked(STSettings.STAnnounce == true)
	ST_SetToolTip(ST_StealingAnnounce,"Announce your stealings into the channels set below")
	ST_StealingOn:SetChecked(STSettings.STEnabled == true)
	ST_SetToolTip(ST_StealingOn,"Enable this wonderful addon")
	ST_StealingPopup:SetChecked(STSettings.STPopup == true)
	ST_SetToolTip(ST_StealingPopup,"Have the frame popup instead of always being shown")
	ST_StealingSound:SetChecked(STSettings.STSound == true)
	ST_SetToolTip(ST_StealingSound,"Make some noise when there is something to steal")
end